package dev.kaderli.magsend

class Utility {
  companion object {
    /**
     * Calculates the CRC-8-AUTOSAR checksum of the given array of bytes.
     *
     * This code is based on the code examples from the following page:
     * http://www.sunshine2k.de/articles/coding/crc/understanding_crc.html
     *
     * The specifics of the algorithm is coming from the AUTOSAR CRC specification:
     * https://www.autosar.org/fileadmin/user_upload/standards/classic/21-11/AUTOSAR_SWS_CRCLibrary.pdf
     *
     * > Utility.crc8Autosar(arrayListOf(0x0, 0x0, 0x0, 0x0)).toString(16)
     * "12"
     * > Utility.crc8Autosar(arrayListOf(0xF2, 0x01, 0x83)).toString(16)
     * "c2"
     * > Utility.crc8Autosar(arrayListOf(0x0F, 0xAA, 0x00, 0x55)).toString(16)
     * "c6"
     * > Utility.crc8Autosar(arrayListOf(0x00, 0xff, 0x55, 0x11)).toString(16)
     * "77"
     * > Utility.crc8Autosar(arrayListOf(0x33, 0x22, 0x55, 0xaa, 0xbb, 0xcc, 0xdd, 0xee, 0xff)).toString(16)
     * "11"
     * > Utility.crc8Autosar(arrayListOf(0x92, 0x6b, 0x55)).toString(16)
     * "33"
     * > Utility.crc8Autosar(arrayListOf(0xff, 0xff, 0xff, 0xff)).toString(16)
     * "6c"
     */
    fun crc8Autosar(bytes: List<Int>): Int {
      val polynomial = 0x2f;
      val xorOut = 0xff;
      var crc = 0xff;

      for (byte in bytes) {
        crc = crc xor byte;

        for(i in 1..8) {
          crc = if ((crc and 0x80) != 0) {
            ((crc shl 1) xor polynomial) and 0xff;
          } else {
            crc shl 1
          }
        }
      }

      return crc xor xorOut;
    }
  }


}
